<?php

namespace Teknisa\EST\Controllers;

use Teknisa\EST\Services\InventarioRotativoService;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\Events\PreDispatch\Listener;
use Zeedhi\Framework\HTTP\Kernel;

class InveRotaValidations extends Listener {

    protected $kernel;
    protected $inveRotaService;
    const INVENTARIO_PENDENTE = "Inventário Rotativo Pendente";
    const DATASET_INVENTARIO_VALIDO = "inventario_valido";
    const INVENTARIO_CEGO_PENDENTE = "Inventário Cego Pendente";
    const CONTAGEM_FISICA_PENDENTE = "Contagem física Pendente";
    const CONTAGEM_FISICA_DIGITACAO_PENDENTE = "Contagem Física Pendente Digitada";
    const REALIZAR_CONTAGEM_ESTOQUE_FISICO = "Realizada a Contagem do Estoque Físico";

    public function __construct(InventarioRotativoService $inveRotaService, Kernel $kernel){
        $this->inveRotaService = $inveRotaService;
        $this->kernel = $kernel;
    }

    /*
     * Método que intercepta toda requisição feita ao backend, realizando
     * a verificação de inventário rotativo de acordo com os parâmetros
     * enviados. Caso a tela esteja parametrizada para verificar estoque
     * e haja inventário rotativo pendente, uma exceção é lançada e tratada
     * nos devidos métodos do frontend, impedindo, assim, o continuamento
     * das funcionalidades normais da tela.
     */
    public function newInveRotaValidation(Request $request, Response $response) {
        $headers = $this->kernel->getHttpRequest()->getHeaders();
        $validaEstoque = $headers->get("ValidaEstoque");
        
        if ($validaEstoque == "true") {
            $cdfilial = $headers->get("Filial");
            $cdfilial = $cdfilial != "false" ? $cdfilial : false;
            
            $isValid = $this->inveRotaService->validateInventarioRotativo($cdfilial);
            if (!$isValid) 
                throw new \Exception(self::INVENTARIO_PENDENTE);
            
            //Blind Inventory Checks
                
            $isValidBlindInvent = $this->inveRotaService->validateBlindInventory($cdfilial);
            if (!$isValidBlindInvent)  
                throw new \Exception(self::INVENTARIO_CEGO_PENDENTE);
            
            $InventCegoInfo = $this->inveRotaService->getInventCegoInfo(true,$cdfilial);
            
            if(!empty($InventCegoInfo)){
                // get operator type
                $hasManagerOrstockist  = $this->inveRotaService->checkOperatorGroups(['GGER','GEST']);
                $hasManager  = $this->inveRotaService->checkOperatorGroups(['GGER']);
                
                // get inventories type
                $hasProgrammedOrInTyping = $this->inveRotaService->checkInventCego($InventCegoInfo,['PR','ED']);
                $hasProgrammed = $this->inveRotaService->checkInventCego($InventCegoInfo,['PR']);
                $hasTyped = $this->inveRotaService->checkInventCego($InventCegoInfo,['DG']);
                
                if($hasManagerOrstockist){
                    if($hasProgrammedOrInTyping) 
                        throw new \Exception(self::CONTAGEM_FISICA_PENDENTE);
                        
                    if($hasTyped && $hasManager) 
                        throw new \Exception(self::CONTAGEM_FISICA_DIGITACAO_PENDENTE);
                    
                    //Checks if there are blind inventories in the current date
                    $InventCegoInfo = $this->inveRotaService->getInventCegoInfo(false,$cdfilial);
                    
                    if(!empty($InventCegoInfo)){
                        $hasProgrammed = $this->inveRotaService->checkInventCego($InventCegoInfo,['PR']);
                        if($hasProgrammed && $hasManagerOrstockist) 
                            throw new \Exception(self::REALIZAR_CONTAGEM_ESTOQUE_FISICO);
                    }
                }
            }
        }
        $response->addDataSet(new DataSet(self::DATASET_INVENTARIO_VALIDO));
    }

    public function preDispatch(Request $request) {
        $headers = $this->kernel->getHttpRequest()->getHeaders();
        $validaEstoque = $headers->get("ValidaEstoque");
        if ($validaEstoque == "true") {
            $cdfilial = $headers->get("Filial");
            if ($cdfilial != "false") {
                $isValid = $this->inveRotaService->validateInventarioRotativo($cdfilial);
            }
            else {
                $isValid = $this->inveRotaService->validateInventarioRotativo();
            }

            if (!$isValid) {
                throw new \Exception(self::INVENTARIO_PENDENTE);
            }
        }
    }
    
    /*
     * Método de validação chamado pelo service no frontend.
     * Como o método de preDispatch vai verificar o inventário rotativo,
     * este método não seria chamado caso houvesse alguma pendência.
     * Como não há a real necessidade de repetir a validação neste método,
     * somente é adicionado um DataSet para indiciar que a verificação foi
     * realizada com sucesso.
     */
    public function validateInventarioRotativo(Request\Row $request, Response $response) {
        $response->addDataSet(new DataSet(self::DATASET_INVENTARIO_VALIDO));
    }
}