<?php

/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

namespace Geral\Logic\PropostaComercial;

/**
 * Description of AceitePropostaComercial
 *
 * @author phillipesilva
 */

use Geral\Util\Ambiente;
use Geral\Util\DateUtil;
use Geral\Util\NovoCodigo;
use Geral\Entities\Cliente;
use Geral\Entities\Inteclie;
use Geral\Entities\Equipetrab;
use Geral\Entities\Propostafas;
use Geral\Entities\Contratoserv;
use Relatione\Entities\Evolcont;
use Faturamento\Entities\Valorlinp;
use Faturamento\Entities\Valorserv;
use Faturamento\Entities\Parcliefas;
use Faturamento\Entities\ParGeralFas;
use Faturamento\Entities\Contratolinp;
use Faturamento\Entities\Propostalinp;
use Faturamento\Entities\Propostaserv;
use Faturamento\Entities\Sistcontlinp;
use Faturamento\Entities\Sistproplinp;
use Faturamento\Entities\Venclusoclie;
use Faturamento\Entities\Venclusoprop;
use Indeterminado\Entities\Vencservclie;
use Indeterminado\Entities\Vencservprop;
use Indeterminado\Entities\Carencialinp;
use Setforms2\Shared\Registry;
use Setforms2\Runtime\Helper\Procedure;
use Setforms2\Runtime\Helper\Procedure\Param;
use Geral\Logic\PropostaComercial\CopiaProspectToCliente;
use Geral\Logic\PropostaComercial\Exception\RestricoesPropostaComercial;

class AceitePropostaComercial {
    public static function aceitaProposta($aceite, $idClieProsp, $idTipoCobrProp) {
        $entityManager = Registry::getEntityManager();
        $db = $entityManager->getConnection();
        $db->beginTransaction();

        try
        {
            $proposta    = Propostafas::findOneByNrproposfasAndNrverpropfas($aceite['NRPROPOSFAS'][''], $aceite['NRVERPROPFAS']['']);
            $cdProspect  = $proposta->getCdProspect();
            $mesReajuste = $proposta->getNrmesreajprop();
            $validacao   = self::validaAceiteProposta($proposta, $aceite);

            if($idClieProsp == 'C') {
                if($mesReajuste == '' || is_null($mesReajuste)) {
                    $mesReajuste = DateUtil::getMes(DateUtil::adicionaIntervalo(DateUtil::getDataAtual(), 1, DateUtil::MES));
                }
            } else {
                $mesReajuste = $aceite['MESREAJUSTE'][''];
            }

            self::cadastrarCliente($proposta, $aceite, $validacao, $mesReajuste);
            self::efetivarAceite($aceite, $validacao, $mesReajuste, DateUtil::getDataAtual(), $idTipoCobrProp);
            self::criarPropostaAtividadeInterna($validacao, $aceite, $mesReajuste, DateUtil::getDataAtual());
            self::gravacaoDeContato($proposta, $validacao, DateUtil::getDataAtual());

            $entityManager->flush();
            $db->commit();
        } catch (\Exception $e) {
            $db->rollback();
            throw $e;
        }

        //Se for prospect é necessário remover os seus registros.
        //Caso não for possível o mesmo ficará inativo
        if($cdProspect) {
            CopiaProspectToCliente::deleteRegistrosProspect($cdProspect, Ambiente::getNrorg());
        }
    }

    public static function validaAceiteProposta(Propostafas $proposta, $aceite) {
        self::verificaDataAceite($proposta, $aceite);
        $validacao = self::verificaInscricaoCliente($aceite, $proposta);
        self::verificaDataCarenciaVencimento($aceite);
        self::verificaDataVencimentoPrimeiraParcela($aceite, $proposta);
        return $validacao;
    }

    public static function cadastrarCliente(Propostafas $proposta, $aceite, $validacao, $mesReajuste) {
        if ($proposta->getIdclienprosp() == 'P') {
            $cdOperador = Ambiente::getCdOperador();
            $nrOrg = Ambiente::getNrorg();

            $dados = array('CDPROSPECT'  => $proposta->getCdprospect(),
                'DTATUAL'     => DateUtil::getDataAtual(),
                'RAZAOSOCIAL' => $aceite['DSRAZAOSOCIAL'][''],
                'COMPLEMENTO' => $aceite['DSCOMPLEMENTO'][''],
                'TPINCRICAO'  => $validacao['TPINSCRICAO'],
                'CDCLIENTE'   => $validacao['INSCRICAO'],
                'NUMJUR'      => $aceite['NUMJUR'][''],
                'NMFANTASIA'  => $aceite['NMFANT'][''],
                'MESREAJUSTE' => $mesReajuste,
                'CDOPERADOR'  => $cdOperador,
                'NRORG'       => $nrOrg);

            CopiaProspectToCliente::copiaDadosToCliente($dados);

            self::updateMesReajuste($aceite['NRPROPOSFAS'][''], $aceite['NRVERPROPFAS'][''], $mesReajuste);
            $operadorResponsavel = Equipetrab::getRepository()->gerenteComercial($proposta->getCdopersoli());
            self::updateGerenteComercialCliente($validacao['INSCRICAO'], $operadorResponsavel, $proposta->getCdopersoli());
            self::updateGerenteComercialParClieFas($validacao['INSCRICAO'], $operadorResponsavel);
        }
    }

    public static function updateGerenteComercialParClieFas($inscricao, $operadorResp) {
        $parcliefas = Parcliefas::findOneByCdclienteAndIdtpfatuclie($inscricao, '5');

        $parcliefas->setCdgerenteimpcli($operadorResp);

        $parcliefas->persist();
    }

    public static function updateGerenteComercialCliente($inscricao, $operadorResp, $operadorSoli) {
        $cliente = Cliente::findOneByCdcliente($inscricao);

        $cliente->setCdgerentecomerc($operadorResp);
        $cliente->setCdagentenegocio($operadorSoli);

        $cliente->persist();
    }

    public static function updateMesReajuste($proposta, $versao, $mesReajuste) {
        /** @var $propostafas \Geral\Entities\Propostafas */
        $propostafas = Propostafas::findOneByNrproposfasAndNrverpropfas($proposta, $versao);

        $propostafas->setNrmesreajprop($mesReajuste);

        $propostafas->persist();
    }

    public static function efetivarAceite($aceite, $validacao, $mesReajuste, $dataAtual, $idTipoCobrProp) {
        $nrorg = Ambiente::getNrorg();
        $cdOperador = Ambiente::getCdOperador();
        $linhaProdutoAceite = Propostalinp::getRepository()->linhaProdutoAceite($aceite['NRPROPOSFAS'][''], $aceite['NRVERPROPFAS']['']);

        $anoAnteriorPosterior = Propostafas::getRepository()->anoAnteriorPosterior($mesReajuste, $aceite['DTACEPROPFAS']['']);
        if ($mesReajuste > $anoAnteriorPosterior[4]) {
            $dtFinAceite = DateUtil::getDataDeString($anoAnteriorPosterior[2], DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            $dtIniAceite = DateUtil::getDataDeString($anoAnteriorPosterior[0], DateUtil::FORMATO_BRASILEIRO_DATAHORA);
        } else {
            $dtFinAceite = DateUtil::getDataDeString($anoAnteriorPosterior[3], DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            $dtIniAceite = DateUtil::getDataDeString($anoAnteriorPosterior[1], DateUtil::FORMATO_BRASILEIRO_DATAHORA);
        }

        /** @var $linha Propostalinp */
        foreach ($linhaProdutoAceite as $linha) {
            $dtcaluslinpp = $linha['DTCALUSLINPP'];

            if ($dtcaluslinpp){
                $dtcaluslinpp = DateUtil::getDataDeString($dtcaluslinpp, DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            } else {
                $dtcaluslinpp = DateUtil::getDataDeString('31/12/1899 00:00:00', DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            }

            $paramServicos = array(
                'ORGANIZACAO'     => $nrorg,
                'DATA_ATUAL'      => $dataAtual,
                'NRPROPOSFAS'     => $linha['NRPROPOSFAS'],
                'NRVERPROPFAS'    => $linha['NRVERPROPFAS'],
                'CDCLIENTE'       => $validacao['INSCRICAO'],
                'CDLINPCLIE'      => $linha['CDLINPPROP'],
                'CDLINPCLIEINFOR' => $linha['CDLINPPROPINFOR'],
                'NMLINPCLIE'      => $linha['NMLINPPROP'],
                'IDTIPLINPC'      => $linha['IDTIPLINPP'],
                'QTMINIMAFATLINP' => $linha['QTMINIMAFATPROP'],
                'NRUSUALINPC'     => $linha['NRUSUALINPP'],
                'IDAMBLINPC'      => $linha['IDAMBLINPP'],
                'IDMPROCLINPC'    => $linha['IDMPROCLINPP'],
                'IDLPROCLINPC'    => $linha['IDLPROCLINPP'],
                'DTCALUSLINPC'    => $dtcaluslinpp,
                'NRMESREAJCONT'   => $mesReajuste,
                'CDLINPPROP'      => $linha['CDLINPPROP']);
            self::insertLinhaProdutoContrato($paramServicos);

            $parcelasLinhaProdutoAceite = Venclusoprop::getRepository()->parcelasLinhaProdutoAceite(
                $aceite['NRPROPOSFAS'][''], $aceite['NRVERPROPFAS'][''], $linha['CDLINPPROP']);

            foreach ($parcelasLinhaProdutoAceite as $parcela) {
                self::insertParcelasLinhaProdutoContrato($validacao['INSCRICAO'], DateUtil::getDataDeString($parcela['DTVELUSOPEDP'], DateUtil::FORMATO_BRASILEIRO_DATAHORA), $parcela['VRVELUSOPEDPP'], $linha['CDLINPPROP'], $nrorg, $dataAtual);
            }

            $aDataAceite = date_parse_from_format("d/m/Y", $aceite['DTACEPROPFAS']['']);
            $aDtcaluslinpp = date_parse_from_format("d/m/Y", $linha['DTCALUSLINPP']);
            while (($aDataAceite['month'] <= $aDtcaluslinpp['month'] && $aDataAceite['year'] <= $aDtcaluslinpp['year']) ||
                ($aDataAceite['month'] > $aDtcaluslinpp['month'] && $aDataAceite['year'] < $aDtcaluslinpp['year'])) {
                $dtCarencia = DateUtil::getDataDeString($aDataAceite['year'] . '/' . $aDataAceite['month'] . '/1', DateUtil::FORMATO_AMERICANO);

                self::insertLinhaProdutoCarencia($validacao['INSCRICAO'], $linha['CDLINPPROP'], $dtCarencia, 'Carência informada na proposta', $dataAtual, $cdOperador, $nrorg);

                $aDataAceite['month'] = $aDataAceite['month'] + 1;
                if ($aDataAceite['month'] == '13') {
                    $aDataAceite['month'] = '01';
                    $aDataAceite['year'] = $aDataAceite['year'] + 1;
                }
            }

            /*
             * Phillipe Silva - 09/10/14
             * O Desconto não estava sendo considerado para os valores da Linha de Produto.
             * Conforme conversado com a Fernanda Prado(Contratos) não seria mais necessário considerar o desconto
             * para os valores da Licença, pois os mesmos seriam retornados posteriormente, mas o valor da manutenção
             * deveria sim ser considerado.
             */

            if(isset($linha['VRDESMENSALPROP']) && $linha['VRDESMENSALPROP'] > 0) {
                $vrMensal = $linha['VRMENSALIDAPROP'] - $linha['VRDESMENSALPROP'];
            } else {
                $vrMensal = $linha['VRMENSALIDAPROP'];
            }

            self::insertLinhaProdutoValores($validacao['INSCRICAO'], $linha['CDLINPPROP'], $dtIniAceite, $dtFinAceite, $linha['VRLICENCUSOPROP'], $vrMensal, $nrorg, $dataAtual);

            $sistemasLinhaProduto = Sistproplinp::getRepository()->sistemasLinhaProduto($aceite['NRPROPOSFAS'][''], $aceite['NRVERPROPFAS'][''], $linha['CDLINPPROP']);

            foreach ($sistemasLinhaProduto as $sistema)
                self::insertSistemasLinhaProduto($validacao['INSCRICAO'], $linha['CDLINPPROP'], $sistema['CDSISTEMA'], $nrorg, $dataAtual);

            if ($linha['IDTIPOPROPFAS'] == 'M')
                self::procedure_FATURA_LICENCAUSO_ACEITE($validacao['INSCRICAO'], $linha['CDLINPPROP'], $cdOperador);
        }

        $servicosProposta = Propostaserv::getRepository()->servicosProposta($aceite['NRPROPOSFAS'], $aceite['NRVERPROPFAS']);

        $isPacote = false;

        if($idTipoCobrProp == 'P') {
            $isPacote = true;
        }

        foreach ($servicosProposta as $servico) {
            if (is_null($servico['IDTIPOCOBRPROP']))
                $servico['IDTIPOCOBRPROP'] = $idTipoCobrProp;

            $paramServicos = array(
                'ORGANIZACAO' => $nrorg,
                'DATA_ATUAL' => $dataAtual,
                'NRPROPOSFAS' => $servico['NRPROPOSFAS'],
                'NRVERPROPFAS' => $servico['NRVERPROPFAS'],
                'CDCLIENTE' => $validacao['INSCRICAO'],
                'CDTIPOSERV' => $servico['CDTIPOSERV'],
                'IDTIPOCOBRCONT' => $servico['IDTIPOCOBRPROP'],
                'QTSERVCONT' => $servico['QTSERVPROP'],
                'VRHORACONT' => $servico['VRHORAPROP'],
                'VRDEHORACONT' => $servico['VRDEHORAPROP'],
                'VRPEACHECONT' => $servico['VRPEACHEPROP'],
                'VREXCEDCONT' => $servico['VREXCEDPROP'],
                'DSLOCALCONT' => $servico['DSLOCALPROP'],
                'NRSERVCONT' => $servico['NRSERVPROP'],
                'NRMESREAJSERV' => $servico['NRMESREAJPROP'],
                'DTIVIGSERV' => $dtIniAceite,
                'DTFVIGSERV' => $dtFinAceite
            );
            self::insertServicosContrato($paramServicos);

            $paramValores = array(
                'DATA_ATUAL' => $dataAtual,
                'ORGANIZACAO' => $nrorg,
                'CDTIPOSERV' => $servico['CDTIPOSERV'],
                'DTIVIGVRSERV' => $dtIniAceite,
                'DTFVIGVRSERV' => $dtFinAceite,
                'VRHORACLIE' => $servico['VRHORAPROP'],
                'VRDEHORACLIE' => $servico['VRDEHORAPROP'],
                'NRPROPOSFAS' => $servico['NRPROPOSFAS'],
                'NRVERPROPFAS' => $servico['NRVERPROPFAS'],
                'NRSERVCONT' => $servico['NRSERVPROP'],
            );
            self::insertValoresServicosContrato($paramValores);
        }

        $where = "NRPROPOSFAS = ?
                  AND NRVERPROPFAS = ?";

        $param = array(
            $aceite['NRPROPOSFAS'][''],
            $aceite['NRVERPROPFAS']['']
        );
        $parcelasServico = Vencservprop::getObjectsByWhere($where, $param);

        /** @var $parcela Vencservprop */
        foreach ($parcelasServico as $parcela) {
            //Novo Codigo sem commit
            $nrVencServClie = NovoCodigo::geraCodigo($nrorg, 'VENCSERVCLIE', 1, 10, false, false);

            $param = array(
                'ORGANIZACAO' => $nrorg,
                'DATA_ATUAL' => $dataAtual,
                'NRVENCSERVCLIE' => $nrVencServClie,
                'CDCLIENTE' => $validacao['INSCRICAO'],
                'DTVENCSERVCLIE' => DateUtil::getDataDeString($parcela->getDtvencservprop(), DateUtil::FORMATO_BRASILEIRO_DATAHORA),
                'VRVENCSERVCLIE' => $parcela->getVrvencservprop(),
                'NRPROPOSFAS' => $aceite['NRPROPOSFAS'][''],
                'NRVERPROPFAS' => $aceite['NRVERPROPFAS']['']
            );
            self::insertParcelasServicosContrato($param);
        }

        if ($isPacote) {
            self::procedure_FATURA_SERVICO_ACEITE($validacao['INSCRICAO'], $cdOperador);
        }

        self::updateDataAceite($aceite['NRPROPOSFAS'][''], $aceite['NRVERPROPFAS'][''], $aceite['DTACEPROPFAS']['']);
    }

    public static function criarPropostaAtividadeInterna($validacao, $aceite, $mesReajuste, $dataAtual) {
        $propostafas = Propostafas::getRepository()->checkPrimeiroAceiteCliente($validacao['INSCRICAO']);

        if ($propostafas) {
            $nrorg = Ambiente::getNrorg();

            $nrCopiaPropFas = NovoCodigo::geraCodigo($nrorg, 'PROPOSTAFAS', 1, 10, false, false);

            $copia        = Propostafas::getRepository()->copiaProposta($aceite['NRPROPOSFAS'][''], $aceite['NRVERPROPFAS']['']);
            $parGeralFas  = ParGeralFas::findOneByNrorg($nrorg);
            $parcliefas   = Parcliefas::findOneByCdclienteAndIdtpfatuclie($copia[0]['CDCLIENTE'], '5');

            if(is_object($parcliefas)) {
                $gerentImpCli = $parcliefas->getCdgerenteimpcli();
            } else {
                $gerentImpCli = '';
            }

            if ($gerentImpCli == '') {
                $cdOperSoli = $copia[0]['CDOPERSOLI'];
            } else {
                $cdOperSoli = $gerentImpCli;
            }

            $param = array(
                'ORGANIZACAO'         => $nrorg,
                'DATA_ATUAL'          => $dataAtual,
                'NRPROPOSFAS'         => $nrCopiaPropFas,
                'NRVERPROPFAS'        => '01',
                'IDCLIENPROSP'        => $copia[0]['IDCLIENPROSP'],
                'CDFILIAL'            => $copia[0]['CDFILIAL'],
                'CDOPERINCL'          => $copia[0]['CDOPERINCL'],
                'DTPROPOSFAS'         => $dataAtual,
                'DTVALPROPFAS'        => DateUtil::adicionaIntervalo($dataAtual, 30),
                'NRCONTATO'           => str_pad($copia[0]['NRCONTATO'], 10, '0', STR_PAD_LEFT),
                'CDCLIENTE'           => $copia[0]['CDCLIENTE'],
                'CDPROSPECT'          => $copia[0]['CDPROSPECT'],
                'DSEMAIPROFAS'        => $copia[0]['DSEMAIPROFAS'],
                'NRFAXPROPFAS'        => $copia[0]['NRFAXPROPFAS'],
                'DSOBSPROPFAS'        => $copia[0]['DSOBSPROPFAS'],
                'IDSOLPROPFAS'        => 'N',
                'CDTPINDICE1'         => $copia[0]['CDTPINDICE1'],
                'IDPERVALOR1'         => $copia[0]['IDPERVALOR1'],
                'CDINDICE1'           => $copia[0]['CDINDICE1'],
                'VRPEINCIND1'         => $copia[0]['VRPEINCIND1'],
                'CDTPINDICE2'         => $copia[0]['CDTPINDICE2'],
                'IDPERVALOR2'         => $copia[0]['IDPERVALOR2'],
                'CDINDICE2'           => $copia[0]['CDINDICE2'],
                'VRPEINCIND2'         => $copia[0]['VRPEINCIND2'],
                'IDTPREAJPROP'        => $copia[0]['IDTPREAJPROP'],
                'CDOPERSOLI'          => $cdOperSoli,
                'NRMESREAJPROP'       => $copia[0]['NRMESREAJPROP'],
                'CDOPERRESP'          => $copia[0]['CDOPERRESP'],
                'DSREFEPROPOSFAS'     => 'Proposta interna - Atividades não-faturadas',
                'CDINTECLIE'          => $copia[0]['CDINTECLIE'],
                'CDINTEPROS'          => $copia[0]['CDINTEPROS'],
                'IDTIPOPROPFAS'       => $copia[0]['IDTIPOPROPFAS'],
                'CDPROCOMERCI'        => $copia[0]['CDPROCOMERCI'],
                'IDTIPOCOBRSERV'      => 'H',
                'VRPACOTESERV'        => $copia[0]['VRPACOTESERV'],
                'IDLIBERAAGENDAMENTO' => $copia[0]['IDLIBERAAGENDAMENTO']
            );
            self::insertCopiaProposta($param);

            $anoAnteriorPosterior = Propostafas::getRepository()->anoAnteriorPosterior($mesReajuste, $aceite['DTACEPROPFAS']['']);
            if ($mesReajuste > $anoAnteriorPosterior[4]) {
                $dtFinAceite = DateUtil::getDataDeString($anoAnteriorPosterior[2], DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                $dtIniAceite = DateUtil::getDataDeString($anoAnteriorPosterior[0], DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            } else {
                $dtFinAceite = DateUtil::getDataDeString($anoAnteriorPosterior[3], DateUtil::FORMATO_BRASILEIRO_DATAHORA);
                $dtIniAceite = DateUtil::getDataDeString($anoAnteriorPosterior[1], DateUtil::FORMATO_BRASILEIRO_DATAHORA);
            }

            $paramContrato = array(
                'ORGANIZACAO'    => $nrorg,
                'DATA_ATUAL'     => $dataAtual,
                'NRPROPOSFAS'    => $nrCopiaPropFas,
                'NRVERPROPFAS'   => '01',
                'CDCLIENTE'      => $copia[0]['CDCLIENTE'],
                'CDTIPOSERV'     => $parGeralFas->getCdtpservpropint(),
                'IDTIPOCOBRCONT' => 'H',
                'QTSERVCONT'     => 999999999,
                'VRHORACONT'     => 1,
                'VRDEHORACONT'   => 0,
                'VRPEACHECONT'   => 1.5,
                'VREXCEDCONT'    => null,
                'DSLOCALCONT'    => '',
                'NRSERVCONT'     => '001',
                'NRMESREAJSERV'  => $copia[0]['NRMESREAJPROP'],
                'DTIVIGSERV'     => $dtIniAceite,
                'DTFVIGSERV'     => $dtFinAceite
            );

            self::insertServicosContrato($paramContrato);

            $paramValoresContrato = array(
                'DATA_ATUAL'   => $dataAtual,
                'ORGANIZACAO'  => $nrorg,
                'CDTIPOSERV'   => $parGeralFas->getCdtpservpropint(),
                'DTIVIGVRSERV' => $dtIniAceite,
                'DTFVIGVRSERV' => $dtFinAceite,
                'VRHORACLIE'   => 1,
                'VRDEHORACLIE' => 0,
                'NRPROPOSFAS'  => $nrCopiaPropFas,
                'NRVERPROPFAS' => '01',
                'NRSERVCONT'   => '001',
            );

            self::insertValoresServicosContrato($paramValoresContrato);

            $param = array(
                'ORGANIZACAO' => Ambiente::getNrorg(),
                'DATA_ATUAL' => $dataAtual,
                'NRPROPOSFAS' => $nrCopiaPropFas,
                'NRVERPROPFAS' => '01',
                'CDTIPOSERV' => $parGeralFas->getCdtpservpropint(),
                'VRHORAPROP' => 1,
                'QTSERVPROP' => 999999999,
                'IDTIPOCOBRPROP' => 'H',
                'VRDEHORAPROP' => 0,
                'VRPEACHEPROP' => 1.5,
                'VREXCEDPROP' => null,
                'DSLOCALPROP' => '',
                'NRSERVPROP' => '001'
            );

            self::insertServicoCopiaProposta($param);

            self::updateDataAceite($nrCopiaPropFas, '01', $aceite['DTACEPROPFAS']['']);
        }
    }

    public static function insertContatoProposta($param) {
        /** @var $evolcont Evolcont */
        $evolcont = new Evolcont();

        $evolcont->setCdfilial($param['CDFILIAL']);
        $evolcont->setNrcontato($param['NRCONTATO']);
        $evolcont->setDthrevolcont($param['DTHREVOLCONT']);
        $evolcont->setCdtipocont($param['CDTIPOCONT']);
        $evolcont->setCdprospect($param['CDPROSPECT']);
        $evolcont->setCdcliente($param['CDCLIENTE']);
        $evolcont->setNmintevocont($param['NMINTEVOCONT']);
        $evolcont->setIdtpevolcont($param['IDTPEVOLCONT']);
        $evolcont->setDsevolcont($param['DSEVOLCONT']);
        $evolcont->setIddescprot($param['IDDESCPROT']);
        $evolcont->setIdageevocont($param['IDAGEEVOCONT']);
        $evolcont->setCdoperevol($param['CDOPEREVOL']);
        $evolcont->setNrseqevolcon($param['NRSEQEVOLCON']);
        $evolcont->setNrseqevolcont($param['NRSEQEVOLCONT']);
        $evolcont->setCdmotievocon($param['CDMOTIEVOCON']);
        $evolcont->setIdsenevocont($param['IDSENEVOCONT']);
        $evolcont->setDthrinclevocont($param['DTHRINCLEVOCONT']);
        $evolcont->setDtinclusao($param['DTINCLUSAO']);
        $evolcont->setIdanexoevolcont($param['IDANEXOEVOLCONT']);
        $evolcont->setNrorg($param['ORGANIZACAO']);
        $evolcont->setNrorginclusao($param['ORGANIZACAO']);
        $evolcont->setCdoperinclusao($param['OPERADOR']);

        $evolcont->persist();
    }

    public static function insertServicoCopiaProposta($param) {
        /** @var $propostaserv Propostaserv */
        $propostaserv = new Propostaserv();

        $propostaserv->setNrproposfas($param['NRPROPOSFAS']);
        $propostaserv->setNrverpropfas($param['NRVERPROPFAS']);
        $propostaserv->setCdtiposerv($param['CDTIPOSERV']);
        $propostaserv->setVrhoraprop($param['VRHORAPROP']);
        $propostaserv->setQtservprop($param['QTSERVPROP']);
        $propostaserv->setIdtipocobrprop($param['IDTIPOCOBRPROP']);
        $propostaserv->setVrdehoraprop($param['VRDEHORAPROP']);
        $propostaserv->setVrpeacheprop($param['VRPEACHEPROP']);
        $propostaserv->setVrexcedprop($param['VREXCEDPROP']);
        $propostaserv->setDslocalprop($param['DSLOCALPROP']);
        $propostaserv->setNrservprop($param['NRSERVPROP']);
        $propostaserv->setNrorg($param['ORGANIZACAO']);
        $propostaserv->setDtinclusao($param['DATA_ATUAL']);

        $propostaserv->persist();
    }

    public static function insertCopiaProposta($param) {
        /** @var $propostafas Propostafas */
        $propostafas = new Propostafas();

        $propostafas->setNrproposfas($param['NRPROPOSFAS']);
        $propostafas->setNrverpropfas($param['NRVERPROPFAS']);
        $propostafas->setIdclienprosp($param['IDCLIENPROSP']);
        $propostafas->setCdfilial($param['CDFILIAL']);
        $propostafas->setCdoperincl($param['CDOPERINCL']);
        $propostafas->setDtproposfas($param['DTPROPOSFAS']);
        $propostafas->setDtvalpropfas($param['DTVALPROPFAS']);
        $propostafas->setNrcontato($param['NRCONTATO']);
        $propostafas->setCdcliente($param['CDCLIENTE']);
        $propostafas->setCdprospect($param['CDPROSPECT']);
        $propostafas->setDsemaiprofas($param['DSEMAIPROFAS']);
        $propostafas->setNrfaxpropfas($param['NRFAXPROPFAS']);
        $propostafas->setDsobspropfas($param['DSOBSPROPFAS']);
        $propostafas->setIdsolpropfas($param['IDSOLPROPFAS']);
        $propostafas->setCdtpindice1($param['CDTPINDICE1']);
        $propostafas->setIdpervalor1($param['IDPERVALOR1']);
        $propostafas->setCdindice1($param['CDINDICE1']);
        $propostafas->setVrpeincind1($param['VRPEINCIND1']);
        $propostafas->setCdtpindice2($param['CDTPINDICE2']);
        $propostafas->setIdpervalor2($param['IDPERVALOR2']);
        $propostafas->setCdindice2($param['CDINDICE2']);
        $propostafas->setVrpeincind2($param['VRPEINCIND2']);
        $propostafas->setIdtpreajprop($param['IDTPREAJPROP']);
        $propostafas->setCdopersoli($param['CDOPERSOLI']);
        $propostafas->setNrmesreajprop($param['NRMESREAJPROP']);
        $propostafas->setCdoperresp($param['CDOPERRESP']);
        $propostafas->setDsrefeproposfas($param['DSREFEPROPOSFAS']);
        $propostafas->setCdinteclie($param['CDINTECLIE']);
        $propostafas->setCdintepros($param['CDINTEPROS']);
        $propostafas->setIdtipopropfas($param['IDTIPOPROPFAS']);
        $propostafas->setCdprocomerci($param['CDPROCOMERCI']);
        $propostafas->setIdtipocobrserv($param['IDTIPOCOBRSERV']);
        $propostafas->setVrpacoteserv($param['VRPACOTESERV']);
        $propostafas->setNrorg($param['ORGANIZACAO']);
        $propostafas->setDtinclusao($param['DATA_ATUAL']);
        $propostafas->setIdliberaagendamento($param['IDLIBERAAGENDAMENTO']);

        $propostafas->persist();

        Registry::getEntityManager()->flush();
    }

    public static function updateDataAceite($proposta, $versao, $dataAceite) {
        $propostafas = Propostafas::findOneByNrproposfasAndNrverpropfas($proposta, $versao);

        $propostafas->setDtacepropfas(DateUtil::getDataDeString($dataAceite));

        $propostafas->persist();
    }

    public static function insertParcelasServicosContrato($param) {
        /** @var $vencservclie Vencservclie */
        $vencservclie = new Vencservclie();

        $vencservclie->setNrvencservclie($param['NRVENCSERVCLIE']);
        $vencservclie->setCdcliente($param['CDCLIENTE']);
        $vencservclie->setDtvencservclie($param['DTVENCSERVCLIE']);
        $vencservclie->setVrvencservclie($param['VRVENCSERVCLIE']);
        $vencservclie->setNrproposfas($param['NRPROPOSFAS']);
        $vencservclie->setNrverpropfas($param['NRVERPROPFAS']);
        $vencservclie->setNrorg($param['ORGANIZACAO']);
        $vencservclie->setDtinclusao($param['DATA_ATUAL']);

        $vencservclie->persist();

        Registry::getEntityManager()->flush();
    }

    public static function insertValoresServicosContrato($param) {
        /** @var $valorserv Valorserv */
        $valorserv = new Valorserv();

        $valorserv->setCdtiposerv($param['CDTIPOSERV']);
        $valorserv->setDtivigvrserv($param['DTIVIGVRSERV']);
        $valorserv->setDtfvigvrserv($param['DTFVIGVRSERV']);
        $valorserv->setVrhoraclie($param['VRHORACLIE']);
        $valorserv->setVrdehoraclie($param['VRDEHORACLIE']);
        $valorserv->setVrdehoraclieant(0);
        $valorserv->setNrproposfas($param['NRPROPOSFAS']);
        $valorserv->setNrverpropfas($param['NRVERPROPFAS']);
        $valorserv->setNrservcont($param['NRSERVCONT']);
        $valorserv->setNrorg($param['ORGANIZACAO']);
        $valorserv->setDtinclusao($param['DATA_ATUAL']);

        $valorserv->persist();
    }

    public static function insertServicosContrato($param) {
        /** @var $contratoserv Contratoserv */
        $contratoserv = new Contratoserv();

        $contratoserv->setNrproposfas($param['NRPROPOSFAS']);
        $contratoserv->setNrverpropfas($param['NRVERPROPFAS']);
        $contratoserv->setCdcliente($param['CDCLIENTE']);
        $contratoserv->setCdtiposerv($param['CDTIPOSERV']);
        $contratoserv->setIdtipocobrcont($param['IDTIPOCOBRCONT']);
        $contratoserv->setQtservcont($param['QTSERVCONT']);
        $contratoserv->setVrhoracont($param['VRHORACONT']);
        $contratoserv->setVrdehoracont($param['VRDEHORACONT']);
        $contratoserv->setVrpeachecont($param['VRPEACHECONT']);
        $contratoserv->setVrexcedcont($param['VREXCEDCONT']);
        $contratoserv->setDslocalcont($param['DSLOCALCONT']);
        $contratoserv->setNrservcont($param['NRSERVCONT']);
        $contratoserv->setNrmesreajserv($param['NRMESREAJSERV']);
        $contratoserv->setDtivigserv($param['DTIVIGSERV']);
        $contratoserv->setDtfvigserv($param['DTFVIGSERV']);
        $contratoserv->setNrorg($param['ORGANIZACAO']);
        $contratoserv->setDtinclusao($param['DATA_ATUAL']);
        $contratoserv->setIdpropservweb('N');

        $contratoserv->persist();

        Registry::getEntityManager()->flush();
    }

    public static function insertSistemasLinhaProduto($inscricao, $cdlinpprop, $cdsistema, $nrorg, $dataAtual) {
        /** @var $sistcontlinp Sistcontlinp */
        $sistcontlinp = new Sistcontlinp();

        $sistcontlinp->setCdcliente($inscricao);
        $sistcontlinp->setCdlinpclie($cdlinpprop);
        $sistcontlinp->setCdsistema($cdsistema);
        $sistcontlinp->setNrorg($nrorg);
        $sistcontlinp->setDtinclusao($dataAtual);

        $sistcontlinp->persist();
    }

    public static function insertLinhaProdutoValores($inscricao, $cdlinpprop, $dtIniAceite, $dtFinAceite, $vrLicenca, $vrManutencao, $nrorg, $dataAtual) {
        /** @var $valorlinp Valorlinp */
        $valorlinp = new Valorlinp();

        $valorlinp->setCdcliente($inscricao);
        $valorlinp->setCdlinpclie($cdlinpprop);
        $valorlinp->setDtivigvrlinp($dtIniAceite);
        $valorlinp->setDtfvigvrlinp($dtFinAceite);
        $valorlinp->setVrlicencusolinp($vrLicenca);
        $valorlinp->setVrmensalidalinp($vrManutencao);
        $valorlinp->setNrorg($nrorg);
        $valorlinp->setDtinclusao($dataAtual);

        $valorlinp->persist();
    }

    public static function insertLinhaProdutoCarencia($inscricao, $cdlinpprop, $dtCarencia, $string, $dataAtual, $cdoperador, $nrorg) {
        /** @var $carencialinp Carencialinp */
        $carencialinp = new Carencialinp();

        $carencialinp->setCdcliente($inscricao);
        $carencialinp->setCdlinpclie($cdlinpprop);
        $carencialinp->setDtmescarenclinp($dtCarencia);
        $carencialinp->setDsobscarenclinp($string);
        $carencialinp->setDtinclusao($dataAtual);
        $carencialinp->setCdoperinclusao($cdoperador);
        $carencialinp->setNrorg($nrorg);

        $carencialinp->persist();
    }

    public static function insertParcelasLinhaProdutoContrato($inscricao, $dtvelusopedp, $vrvelusopedp, $cdlinpprop, $nrorg, $dataAtual) {
        /** @var $contratolinp Venclusoclie */
        $venclusoclie = new Venclusoclie();

        $venclusoclie->setCdcliente($inscricao);
        $venclusoclie->setDtvelusopedc($dtvelusopedp);
        $venclusoclie->setVrvelusopedc($vrvelusopedp);
        $venclusoclie->setCdlinpclie($cdlinpprop);
        $venclusoclie->setNrorg($nrorg);
        $venclusoclie->setDtinclusao($dataAtual);

        $venclusoclie->persist();

        Registry::getEntityManager()->flush();
    }

    public static function insertLinhaProdutoContrato($param) {
        /** @var $contratolinp \Faturamento\Entities\Contratolinp */
        $contratolinp = new Contratolinp();

        $contratolinp->setNrorg($param['ORGANIZACAO']);
        $contratolinp->setNrproposfas($param['NRPROPOSFAS']);
        $contratolinp->setNrverpropfas($param['NRVERPROPFAS']);
        $contratolinp->setCdcliente($param['CDCLIENTE']);
        $contratolinp->setCdlinpclie($param['CDLINPPROP']);
        $contratolinp->setCdlinpclieinfor($param['CDLINPCLIEINFOR']);
        $contratolinp->setNmlinpclie($param['NMLINPCLIE']);
        $contratolinp->setIdtiplinpc($param['IDTIPLINPC']);
        $contratolinp->setQtminimafatlinp($param['QTMINIMAFATLINP']);
        $contratolinp->setNrusualinpc($param['NRUSUALINPC']);
        $contratolinp->setIdamblinpc($param['IDAMBLINPC']);
        $contratolinp->setIdmproclinpc($param['IDMPROCLINPC']);
        $contratolinp->setIdlproclinpc($param['IDLPROCLINPC']);
        $contratolinp->setDtcaluslinpc($param['DTCALUSLINPC']);
        $contratolinp->setNrmesreajcont($param['NRMESREAJCONT']);
        $contratolinp->setDtinclusao($param['DATA_ATUAL']);
        $contratolinp->setIdbloqlinp('N');

        $contratolinp->persist();

        Registry::getEntityManager()->flush();
    }

    public static function procedure_FATURA_SERVICO_ACEITE($inscricao, $cdoperador) {
        $procedure = new Procedure('FATURA_SERVICO_ACEITE');
        $procedure->addParam(
            new Param(
                'P_CDCLIENTE', Param::PARAM_INPUT, $inscricao, Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_CDOPERAPUR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_COMMIT', Param::PARAM_INPUT, '0', Param::PARAM_TYPE_STR
            )
        );
        $procedure->call();
    }

    public static function procedure_FATURA_LICENCAUSO_ACEITE($inscricao, $cdlinpprop, $cdoperador) {
        $procedure = new Procedure('FATURA_LICENCAUSO_ACEITE');
        $procedure->addParam(
            new Param(
                'P_CDCLIENTE', Param::PARAM_INPUT, $inscricao, Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_CDLINPCLIE', Param::PARAM_INPUT, $cdlinpprop, Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_CDOPERAPUR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR
            )
        );
        $procedure->addParam(
            new Param(
                'P_COMMIT', Param::PARAM_INPUT, '0', Param::PARAM_TYPE_STR
            )
        );
        $procedure->call();
    }

    public static function gravacaoDeContato(Propostafas $proposta, $validacao, $dataAtual) {
        if ($proposta->getCdinteclie() != '') {
            $cdInteClie = $proposta->getCdinteclie();
        } else {
            $cdInteClie = $proposta->getCdintepros();
        }

        if ($proposta->getNrcontato()) {
            $interlocutorCliente     = Inteclie::findOneByCdclienteAndCdinteclie($validacao['INSCRICAO'], $cdInteClie);
            $numeroSequencialContato = Evolcont::getRepository()->numeroSequencialContato($proposta->getNrcontato());
            $numeroSeqContato        = Evolcont::getRepository()->consultaProxCodigoSeqEvol();

            if ($interlocutorCliente) {
                $interlocutor = $interlocutorCliente->getNminteclie();
            } else {
                $interlocutor = '';
            }
            if (isset($numeroSequencialContato[0])) {
                if ($numeroSequencialContato[0] != '') {
                    $seqContato = $numeroSequencialContato[0] + 1;
                } else {
                    $seqContato = '01';
                }
            }

            if($seqContato > 99) {
                $seqContato = '01';
            }

            $param = array(
                'ORGANIZACAO' => Ambiente::getNrorg(),
                'OPERADOR' => Ambiente::getCdOperador(),
                'CDFILIAL' => $proposta->getCdfilial(),
                'NRCONTATO' => $proposta->getNrcontato(),
                'DTHREVOLCONT' => $dataAtual,
                'CDTIPOCONT' => '30',
                'CDPROSPECT' => null,
                'CDCLIENTE' => $validacao['INSCRICAO'],
                'NMINTEVOCONT' => $interlocutor,
                'IDTPEVOLCONT' => 'C',
                'DSEVOLCONT' => 'Recebido aceite da Proposta Comercial ' . $proposta->getNrproposfas() . '-' . $proposta->getNrverpropfas(),
                'IDDESCPROT' => 'S',
                'IDAGEEVOCONT' => 'N',
                'CDOPEREVOL' => $proposta->getCdopersoli(),
                'NRSEQEVOLCON' => $seqContato,
                'NRSEQEVOLCONT' => $numeroSeqContato[0]['NRSEQEVOLCONT'],
                'CDMOTIEVOCON' => '00004',
                'IDSENEVOCONT' => 'E',
                'DTHRINCLEVOCONT' => $dataAtual,
                'DTINCLUSAO' => $dataAtual,
                'IDANEXOEVOLCONT' => 'S'
            );

            self::insertContatoProposta($param);
        }
    }

    public static function ajustaTamanhoString($string, $tam) {
        return strlen($string) < $tam ? str_pad($string, $tam, '0', STR_PAD_LEFT) : substr($string, -$tam);
    }

    public static function verificaDataAceite(Propostafas $proposta, $aceite) {
        if (DateUtil::comparaDatas(DateUtil::getDataDeString($aceite['DTACEPROPFAS']['']), $proposta->getDtproposfas()))
            RestricoesPropostaComercial::dataAceiteMenorDataProposta();
    }

    public static function verificaInscricaoCliente($aceite, Propostafas $proposta) {
        $tpInscricao = '';
        if ($proposta->getIdclienprosp() == 'P') {
            $inscricao = self::ajustaInscricao($aceite['NUMJUR']['']);
            $prospect = \Geral\Entities\Prospect::findOneByCdprospect($proposta->getCdprospect());
            $tpInscricao = $prospect->getIdtpijurpros();

            /** @var $cliente Cliente */
            $cliente = self::inscricaoCliente($inscricao);
            if ($cliente instanceof \Setforms2\Shared\ORM\BasicEntity)
                RestricoesPropostaComercial::inscricaoJaUtilizada();
        } else
            $inscricao = $proposta->getCdcliente();

        $validacao = array('TPINSCRICAO' => $tpInscricao,
            'INSCRICAO'   => $inscricao);

        return $validacao;
    }

    public static function verificaDataCarenciaVencimento($aceite) {
        $linhaProduto = self::linhaProduto($aceite['NRPROPOSFAS'][''], $aceite['NRVERPROPFAS']['']);

        $linhaProdutoErroCarencia = array();
        $linhaProdutoErroVencimento = array();

        foreach ($linhaProduto as $linha) {
            /** @var $linha Propostalinp */
            if ($linha->getDtcaluslinpp() != null && DateUtil::comparaDatas($linha->getDtcaluslinpp(), DateUtil::getDataDeString($aceite['DTACEPROPFAS'][''])))
                $linhaProdutoErroCarencia[] = $linha->getNmlinpprop();

            $parcelasLinhaProduto = Venclusoprop::getRepository()->parcelasLinhaProduto($aceite['NRPROPOSFAS'][''], $aceite['NRVERPROPFAS'][''], $linha->getCdlinpprop());

            if ($parcelasLinhaProduto != null && DateUtil::comparaDatas(DateUtil::getDataDeString($parcelasLinhaProduto, DateUtil::FORMATO_BRASILEIRO_DATAHORA), DateUtil::getDataDeString($aceite['DTACEPROPFAS'][''])))
                $linhaProdutoErroVencimento[] = $linha->getNmlinpprop();
        }

//        Phillipe Silva - 28/08/14
//        A carência não será mais considerada durante a efetivação do aceite.
//        if (count($linhaProdutoErroCarencia) != 0)
//            RestricoesPropostaComercial::dataCarenciaMenorDataAceite($linhaProdutoErroCarencia);
        if (count($linhaProdutoErroVencimento) != 0)
            RestricoesPropostaComercial::dataVencimentoMenorDataAceite($linhaProdutoErroVencimento);
    }

    public static function verificaDataVencimentoPrimeiraParcela($aceite, Propostafas $proposta) {
        if ($proposta->getIdtipocobrserv() == 'P') {
            $parcelasServicos = Vencservprop::getRepository()->parcelasServicos($aceite['NRPROPOSFAS'][''], $aceite['NRVERPROPFAS']['']);

            if ($parcelasServicos != null && DateUtil::comparaDatas(DateUtil::getDataDeString($parcelasServicos, DateUtil::FORMATO_BRASILEIRO_DATAHORA), DateUtil::getDataDeString($aceite['DTACEPROPFAS'][''])))
                RestricoesPropostaComercial::dataVencimentoPrimeiraParcela();
        }
    }

    public static function ajustaInscricao($inscricao) {
        $aux = $inscricao;
        $inscricao = '';

        if (!empty($aux)) {
            for ($i = 0; $i < strlen($aux); $i++) {
                if ($aux[$i] >= '0' || $aux[$i] <= '9')
                    $inscricao = $inscricao . $aux[$i];
            }
            $inscricao = trim($inscricao);
            $tam = strlen($inscricao);

            switch ($tam) {
                case 14:
                    if (!self::validateCNPJ($inscricao)) {
                        RestricoesPropostaComercial::cnpjInvalido();
                    }
                    $result = 'J';
                    break;
                case 11:
                    if (!self::validateCPF($inscricao)) {
                        RestricoesPropostaComercial::cpfInvalido();
                    }
                    $result = 'F';
                    break;
                default:
                    if ($tam < 6) {
                        if (!empty($inscricao)) {
                            if ($tam != 5) {
                                $inscricao = self::ajustaTamanhoString($inscricao, 5);
                                $result = 'O';
                            }
                        }
                    } else {
                        if (!empty($inscricao)) {
                            if ($tam != 20) {
                                $inscricao = self::ajustaTamanhoString($inscricao, 20);
                            }
                            $result = 'I';
                        }
                    }
                    break;
            }
            $result = $result . $inscricao;
        }
        return $result;
    }

    public static function inscricaoCliente($inscricao) {
        $where = "CDCLIENTE = :P_CDCLIENTE
                  OR (IDTPIJURCLIE = SUBSTR(:P_CDCLIENTE,1,1) AND NRINSJURCLIE = TRIM(SUBSTR(:P_CDCLIENTE,2)))";

        $param = array(
            ":P_CDCLIENTE" => $inscricao
        );

        return Cliente::getObjectByWhere($where, $param);
    }

    public static function linhaProduto($proposta, $versao) {
        $where = "NRPROPOSFAS       = ?
                  AND NRVERPROPFAS  = ?
                  AND NRORG         = 1";

        $param = array(
            $proposta,
            $versao
        );

        return Propostalinp::getObjectsByWhere($where, $param);
    }

    public static function validateCPF($cpf) {
        $nulos = array("12345678909", "11111111111", "22222222222", "33333333333",
            "44444444444", "55555555555", "66666666666", "77777777777",
            "88888888888", "99999999999", "00000000000");
        $cpf = preg_replace("/[^0-9]/", "", $cpf);
        if (!(preg_match("/[0-9]/", $cpf)))
            return false;
        if (in_array($cpf, $nulos))
            return false;
        if (strlen($cpf) != 11) {
            return false;
        }
        $acum = 0;
        for ($i = 0; $i < 9; $i++) {
            $acum += $cpf[$i] * (10 - $i);
        }

        $x = $acum % 11;
        $acum = ($x > 1) ? (11 - $x) : 0;
        if ($acum != $cpf[9]) {
            return false;
        }
        $acum = false;
        for ($i = 0; $i < 10; $i++) {
            $acum += $cpf[$i] * (11 - $i);
        }

        $x = $acum % 11;
        $acum = ($x > 1) ? (11 - $x) : 0;
        if ($acum != $cpf[10]) {
            return false;
        }
        return true;
    }

    public static function validateCNPJ($cnpj) {
        $cnpj = preg_replace("/[^0-9]/", "", $cnpj);
        if (strlen($cnpj) != 14) {
            return false;
        }

        $soma = 0;

        $soma += ($cnpj[0] * 5);
        $soma += ($cnpj[1] * 4);
        $soma += ($cnpj[2] * 3);
        $soma += ($cnpj[3] * 2);
        $soma += ($cnpj[4] * 9);
        $soma += ($cnpj[5] * 8);
        $soma += ($cnpj[6] * 7);
        $soma += ($cnpj[7] * 6);
        $soma += ($cnpj[8] * 5);
        $soma += ($cnpj[9] * 4);
        $soma += ($cnpj[10] * 3);
        $soma += ($cnpj[11] * 2);

        $d1 = $soma % 11;
        $d1 = $d1 < 2 ? 0 : 11 - $d1;

        $soma = 0;
        $soma += ($cnpj[0] * 6);
        $soma += ($cnpj[1] * 5);
        $soma += ($cnpj[2] * 4);
        $soma += ($cnpj[3] * 3);
        $soma += ($cnpj[4] * 2);
        $soma += ($cnpj[5] * 9);
        $soma += ($cnpj[6] * 8);
        $soma += ($cnpj[7] * 7);
        $soma += ($cnpj[8] * 6);
        $soma += ($cnpj[9] * 5);
        $soma += ($cnpj[10] * 4);
        $soma += ($cnpj[11] * 3);
        $soma += ($cnpj[12] * 2);


        $d2 = $soma % 11;
        $d2 = $d2 < 2 ? 0 : 11 - $d2;

        if ($cnpj[12] == $d1 && $cnpj[13] == $d2) {
            return true;
        } else {
            return false;
        }
    }
}